from django.apps import AppConfig


class WagtailImageCaptionsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "wagtailimagecaptions"

    def ready(self):
        # Implicitly connect signal handlers decorated with @receiver.
        from . import signals  # noqa



from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.utils.text import Truncator
from wagtail.images import get_image_model_string

from .services import parse_iptc

IMAGE_MODEL = get_image_model_string()


@receiver(pre_save, sender=IMAGE_MODEL)
def parse_image_meta(sender, **kwargs):
    """
    Parses image meta data from a tif or jpeg and populates the corresponding
    fields of the image model.
    """
    instance = kwargs["instance"]

    # If we have an ID, don't parse again.
    if instance.id is not None:
        return

    meta_dict = parse_iptc(instance.file)

    # Add the meta data to the fields.
    if title := meta_dict.get("headline", ""):
        trimmed_title = Truncator(title.strip()).chars(255)
        instance.title = trimmed_title
        instance.alt = trimmed_title

    if credit := meta_dict.get("credit", ""):
        trimmed_credit = Truncator(credit.strip()).chars(255)
        instance.credit = trimmed_credit

    if caption := meta_dict.get("caption", ""):
        trimmed_caption = Truncator(caption.strip()).chars(255)
        instance.caption = trimmed_caption

    instance.iptc_data = meta_dict



from django.db import models

from wagtail.fields import RichTextField
from wagtail.images.models import Image, AbstractImage, AbstractRendition
from wagtail.search import index


class CaptionedImage(AbstractImage):
    alt = models.CharField(
        max_length=255,
        blank=True,
        help_text="Set the image alt text used for accessibility.",
    )
    caption = RichTextField(features=["bold", "italic"], null=True, blank=True, help_text="Set the image caption.")
    credit = models.CharField(max_length=255, blank=True, help_text="Set the image credit.")
    iptc_data = models.JSONField(null=True, blank=True)

    admin_form_fields = Image.admin_form_fields + (
        "credit",
        "alt",
        "caption",
    )

    search_fields = AbstractImage.search_fields + [
        index.SearchField("caption", partial_match=True, boost=10),
    ]

    def default_alt_text(self):
        """ Return our stored alt value, otherwise Wagtail defaults to the title. """
        if self.alt:
            return self.alt
        return super().default_alt_text


class ExtendedRendition(AbstractRendition):
    image = models.ForeignKey(CaptionedImage, on_delete=models.CASCADE, related_name="renditions")

    class Meta:
        unique_together = (("image", "filter_spec", "focal_point_key"),)
